/*
 *  Copyright (c) xlightweb.org, 2008 - 2009. All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Please refer to the LGPL license at: http://www.gnu.org/copyleft/lesser.txt
 * The latest copy of this software may be found on http://www.xlightweb.org/
 */
package org.xlightweb;

import java.net.URL;
import java.util.Collections;
import java.util.Enumeration;
import java.util.List;
import java.util.Set;




 
/**
 * Provides a convenient implementation of the HttpRequestHeader interface that can 
 * be subclassed by developers wishing to adapt the request header. This class implements 
 * the Wrapper or Decorator pattern. Methods default to calling through to the 
 * wrapped request header object.   
 * 
 * @author grro@xlightweb.org
 */
public class HttpRequestHeaderWrapper implements IHttpRequestHeader {

	
	private final IHttpRequestHeader requestHeader;
	

	/**
	 * constructor 
	 * 
	 * @param request the request header to wrap
	 */
	public HttpRequestHeaderWrapper(IHttpRequestHeader requestHeader) {
		this.requestHeader = requestHeader;
	}
	
	
	protected IHttpRequestHeader getWrappedRequestHeader() {
		return requestHeader;
	}
	

	/**
	 * {@inheritDoc}
	 */
	public Boolean getBooleanParameter(String name) {
		return requestHeader.getBooleanParameter(name);
	}


	/**
	 * {@inheritDoc}
	 */
	public boolean getBooleanParameter(String name, boolean defaultVal) {
		return requestHeader.getBooleanParameter(name, defaultVal);
	}


	/**
	 * {@inheritDoc}
	 */
	public Double getDoubleParameter(String name) throws BadMessageException {
		return requestHeader.getDoubleParameter(name);
	}

	/**
	 * {@inheritDoc}
	 */
	public double getDoubleParameter(String name, double defaultVal) {
		return requestHeader.getDoubleParameter(name, defaultVal);
	}

	/**
	 * {@inheritDoc}
	 */
	public Float getFloatParameter(String name) throws BadMessageException {
		return requestHeader.getFloatParameter(name);
	}

	/**
	 * {@inheritDoc}
	 */
	public float getFloatParameter(String name, float defaultVal) {
		return requestHeader.getFloatParameter(name, defaultVal);
	}

	/**
	 * {@inheritDoc}
	 */
	public Integer getIntParameter(String name) throws BadMessageException {
		return requestHeader.getIntParameter(name);
	}


	/**
	 * {@inheritDoc}
	 */
	public int getIntParameter(String name, int defaultVal) {
		return requestHeader.getIntParameter(name, defaultVal);
	}


	/**
	 * {@inheritDoc}
	 */
	public Long getLongParameter(String name) throws BadMessageException {
		return requestHeader.getLongParameter(name);
	}


	/**
	 * {@inheritDoc}
	 */
	public long getLongParameter(String name, long defaultVal) {
		return requestHeader.getLongParameter(name, defaultVal);
	}


	/**
	 * {@inheritDoc}
	 */
	public String getMethod() {
		return requestHeader.getMethod();
	}


	
    /**
     * {@inheritDoc}
     */
	public String getMatrixParameter(String name) {
	    return requestHeader.getMatrixParameter(name);
	}
	
	
	
    /**
     * {@inheritDoc}
     */
	public Set<String> getMatrixParameterNameSet() {
	   return requestHeader.getMatrixParameterNameSet();
	}


	
    /**
     * {@inheritDoc}
     */
	public String[] getMatrixParameterValues(String name) {
	    return requestHeader.getParameterValues(name);
	}

	
    /**
     * {@inheritDoc}
     */
	public void setMatrixParameter(String parameterName, String parameterValue) {
	    requestHeader.setMatrixParameter(parameterName, parameterValue);
	}
	
	
	/**
	 * {@inheritDoc}
	 */
	public void addMatrixParameter(String parameterName, String parameterValue) {
	    requestHeader.addMatrixParameter(parameterName, parameterValue);
	}
	
	
	/**
     * {@inheritDoc}
     */
	public void removeMatrixParameter(String parameterName) {
	    requestHeader.removeMatrixParameter(parameterName);
	}
	
	
	/**
	 * {@inheritDoc}
	 */
	public void removeParameter(String parameterName) {
	    requestHeader.removeParameter(parameterName);
	}

	/**
	 * {@inheritDoc}
	 */
	public String getParameter(String name) {
		return requestHeader.getParameter(name);
	}


	/**
	 * {@inheritDoc}
	 */
	public Set<String> getParameterNameSet() {
		return requestHeader.getParameterNameSet();
	}


	/**
	 * {@inheritDoc}
	 */
	@SuppressWarnings("unchecked")
	public Enumeration getParameterNames() {
		return Collections.enumeration(getParameterNameSet());
	}

	
	/**
	 * {@inheritDoc}
	 */
	
	public String[] getParameterValues(String name) {
		return requestHeader.getParameterValues(name);
	}

	
	/**
	 * {@inheritDoc}
	 */
	public String getQueryString() {
		return requestHeader.getQueryString();
	}


	/**
	 * {@inheritDoc}
	 */
	public String getRemoteAddr() {
		return requestHeader.getRemoteAddr();
	}


	/**
	 * {@inheritDoc}
	 */
	public String getRemoteHost() {
		return requestHeader.getRemoteHost();
	}


	/**
	 * {@inheritDoc}
	 */
	public int getRemotePort() {
		return requestHeader.getRemotePort();
	}


	/**
	 * {@inheritDoc}
	 */
	public String getRequestURI() {
		return requestHeader.getRequestURI();
	}

	
	/** 
	 * {@inheritDoc}
	 */ 
	public void setRequestURI(String requestUri) {
		requestHeader.setRequestURI(requestUri);
	}

	/**
	 * {@inheritDoc}
	 */
	public URL getRequestUrl() {
		return requestHeader.getRequestUrl();
	}


	/**
	 * {@inheritDoc}
	 */
	public boolean getRequiredBooleanParameter(String name) throws BadMessageException {
		return requestHeader.getRequiredBooleanParameter(name);
	}


	/**
	 * {@inheritDoc}
	 */
	public double getRequiredDoubleParameter(String name) throws BadMessageException {
		return requestHeader.getRequiredDoubleParameter(name);
	}


	/**
	 * {@inheritDoc}
	 */
	public float getRequiredFloatParameter(String name) throws BadMessageException {
		return requestHeader.getRequiredFloatParameter(name);
	}


	/**
	 * {@inheritDoc}
	 */
	public int getRequiredIntParameter(String name) throws BadMessageException {
		return requestHeader.getRequiredIntParameter(name);
	}


	/**
	 * {@inheritDoc}
	 */
	public long getRequiredLongParameter(String name) throws BadMessageException {
		return requestHeader.getRequiredLongParameter(name);
	}


	/**
	 * {@inheritDoc}
	 */
	public String getRequiredStringParameter(String name) throws BadMessageException {
		return requestHeader.getRequiredStringParameter(name);
	}


	/**
	 * {@inheritDoc}
	 */
	public String getServerName() {
		return requestHeader.getServerName();
	}


	/**
	 * {@inheritDoc}
	 */
	public int getServerPort() {
		return requestHeader.getServerPort();
	}

	
	/**
	 * {@inheritDoc}
	 */
	public boolean isSecure() {
		return requestHeader.isSecure();
	}


	/**
	 * {@inheritDoc}
	 */
	public void setMethod(String method) {
		requestHeader.setMethod(method);		
	}


	/**
	 * {@inheritDoc}
	 */
	public void setParameter(String parameterName, String parameterValue) {
		requestHeader.setParameter(parameterName, parameterValue);
	}


	/**
     * {@inheritDoc}
     */
	public void addParameter(String parameterName, String parameterValue) {
	    requestHeader.addParameter(parameterName, parameterValue);
	}
	

	/**
	 * {@inheritDoc}
	 */
	public void setRequestUrl(URL url) {
		requestHeader.setRequestUrl(url);		
	}


	/**
	 * {@inheritDoc}
	 */
	public void addHeader(String headername, String headervalue) {
		requestHeader.addHeader(headername, headervalue);		
	}


	/**
	 * {@inheritDoc}
	 */
	public void addHeaderLine(String line) {
		requestHeader.addHeaderLine(line);		
	}

	
	/**
     * {@inheritDoc}
     */
	public void addHeaderlines(String... lines) {
	    requestHeader.addHeaderlines(lines);
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean containsHeader(String headername) {
		return requestHeader.containsHeader(headername);
	}


	/**
	 * {@inheritDoc}
	 */
	public Object getAttribute(String name) {
		return requestHeader.getAttribute(name);
	}


	/**
	 * {@inheritDoc}
	 */
	public Set<String> getAttributeNameSet() {
		return requestHeader.getAttributeNameSet();
	}


	/**
	 * {@inheritDoc}
	 */
	@SuppressWarnings("unchecked")
	public Enumeration getAttributeNames() {
		return requestHeader.getAttributeNames();
	}


	/**
	 * {@inheritDoc}
	 */
	public String getCharacterEncoding() {
		return requestHeader.getCharacterEncoding();
	}


	/**
	 * {@inheritDoc}
	 */
	public int getContentLength() {
		return requestHeader.getContentLength();
	}


	/**
	 * {@inheritDoc}
	 */
	public String getContentType() {
		return requestHeader.getContentType();
	}
	

	/**
	 * {@inheritDoc}
	 */
	public List<ContentType> getAccept() {
		return requestHeader.getAccept();
	}

	/**
     * {@inheritDoc}
     */
	public String getScheme() {
	    return requestHeader.getScheme();
	}
	
	/**
     * {@inheritDoc}
     */
	public String getDisposition() {
	    return requestHeader.getDisposition();
	}

	
	/**
     * {@inheritDoc}
     */
	public String getDispositionType() {
	    return requestHeader.getDispositionType();
	}

	
	/**
     * {@inheritDoc}
     */
	public String getDispositionParam(String name) {
	    return requestHeader.getDispositionParam(name);
	}
	
	
	/**
	 * {@inheritDoc}
	 */
	public String getHeader(String headername) {
		return requestHeader.getHeader(headername);
	}

	
	/**
	 * {@inheritDoc}
	 */
	public String getConnection() {
		return requestHeader.getConnection();
	}
	
	
	/**
	 * {@inheritDoc}
	 */
	public String getHost() {
		return requestHeader.getHost();
	}
	
	
	/**
	 * {@inheritDoc}
	 */
	public String getKeepAlive() {
		return requestHeader.getKeepAlive();
	}

	
	/**
	 * {@inheritDoc}
	 */
	public String getUpgrade() {
		return requestHeader.getKeepAlive();
	}
	
	
	/**
	 * {@inheritDoc}
	 */
	public String getUserAgent() {
		return requestHeader.getUserAgent();
	}
	
	
	/**
	 * {@inheritDoc}
	 */
	public void setUpgrade(String upgrade) {
		requestHeader.setUpgrade(upgrade);
	}
	
	
	/**
	 * {@inheritDoc}
	 */
	public void setConnection(String connection) {
		requestHeader.setConnection(connection);
	}
	
	
	/**
	 * {@inheritDoc}
	 */
	public void setHost(String host) {
		requestHeader.setHost(host);
	}
	
	
	/**
	 * {@inheritDoc}
	 */
	public void setKeepAlive(String keepAlive) {
		requestHeader.setKeepAlive(keepAlive);
	}
	
	
	
	/**
	 * {@inheritDoc}
	 */
	public void setUserAgent(String userAgent) {
		requestHeader.setUserAgent(userAgent);
	}
	
	
	/**
	 * {@inheritDoc}
	 */
	public IHttpRequestHeader copy() {
		return new HttpRequestHeaderWrapper(this);
	}
	
	
	/**
	 * {@inheritDoc}
	 */
	public void copyHeaderFrom(IHttpHeader otherHeader, String... upperExcludenames) {
		requestHeader.copyHeaderFrom(otherHeader, upperExcludenames);
	}
	
	
	/**
	 * {@inheritDoc}
	 */
	public List<String> getHeaderList(String headername) {
		return requestHeader.getHeaderList(headername);
	}


	/**
	 * {@inheritDoc}
	 */
	public Set<String> getHeaderNameSet() {
		return requestHeader.getHeaderNameSet();
	}


	/**
	 * {@inheritDoc}
	 */
	@SuppressWarnings("unchecked")
	public Enumeration getHeaderNames() {
		return requestHeader.getHeaderNames();
	}


	/**
	 * {@inheritDoc}
	 */
	@SuppressWarnings("unchecked")
	public Enumeration getHeaders(String headername) {
		return requestHeader.getHeaders(headername);
	}



	/**
	 * {@inheritDoc}
	 */
	public String getProtocol() {
		return requestHeader.getProtocol();
	}


	/**
	 * {@inheritDoc}
	 */
	public String getProtocolVersion() {
		return requestHeader.getProtocolVersion();
	}


	/**
	 * {@inheritDoc}
	 */
	public String getTransferEncoding() {
		return requestHeader.getTransferEncoding();
	}



	/**
	 * {@inheritDoc}
	 */
	public void removeHeader(String headername) {
		requestHeader.removeHeader(headername);		
	}
	

	/**
	 * {@inheritDoc}
	 */
	public void removeHopByHopHeaders() {
	    requestHeader.removeHopByHopHeaders();
	}


	/**
	 * {@inheritDoc}
	 */
	public void setAttribute(String name, Object o) {
		requestHeader.setAttribute(name, o);		
	}


	/**
	 * {@inheritDoc}
	 */
	public void setContentLength(int length) {
		requestHeader.setContentLength(length);		
	}


	/**
	 * {@inheritDoc}
	 */
	public void setContentType(String type) {
		requestHeader.setContentType(type);		
	}
	
	/**
	 * {@inheritDoc}
	 */
	public void setHeader(String headername, String headervalue) {
		requestHeader.setHeader(headername, headervalue);		
	}


	/**
	 * {@inheritDoc}
	 */
	public void setTransferEncoding(String transferEncoding) {
		requestHeader.setTransferEncoding(transferEncoding);		
	}	
	
	@Override
	public String toString() {
		return requestHeader.toString();
	}
}
